#import "BWBorkSortController.h"
#import "BWLog.h"
#import "BWFileList.h"
#import "BWScaledImageView.h"
#import "BWBorkSortPrefPanel.h"
#import "BWBorkSortPreferences.h"

static const char *cvsid = "$Id: BWBorkSortController.m,v 1.2 2002/12/12 02:37:20 markd Exp $";
static BWLogModule *logger;

@implementation BWBorkSortController


//
// ----- HouseKeeping
//
+ (void) initialize
{
    [super initialize];

    logger = [BWLogModule registerModule: @"BWBorkSortController"
			  withCVSId: cvsid];
} // initialize


- (id) init
{
    BWLog (logger, BWTrace, @"init");

    if (self = [super init]) {
	NSString *imagePath;
	NSBundle *bundleUp;

	bundleUp = [NSBundle bundleForClass: [self class]];

	imagePath = [bundleUp pathForResource: @"cant-open-file"  
			      ofType: @"jpg"];
	noImageImage = [[NSImage alloc] initWithContentsOfFile: imagePath];
	[noImageImage retain];
    }

    return (self);

} // init


- (void) updateUI
{
    int row;
    int count;
    
    BWLog (logger, BWVerbose, @"updating UI");

    row = [fileList selectedRow];

    count = [files count];

    if (row == -1) {

	BWLog (logger, BWVerbose, @"no selected row.  disabling buttons, clearing path label and image");

	[nukeButton setEnabled: NO];
	[keepButton setEnabled: NO];
	[imageView setImage: nil];
	if (count > 0) {
	    [pathLabel setStringValue: @""];
	} else {
	    [pathLabel setStringValue: originalPathLabelValue];
	}
	
    } else {
	BWFileEntry *entry;
	entry = [files getFileAt: row];

	BWLog (logger, BWVerbose, 
	       @"row %d selected.  enabling buttons and using file entry %@", 
	       row, entry);

	[nukeButton setEnabled: YES];
	[keepButton setEnabled: YES];

	[self displayImage: entry];
	[[imageView window] setTitle: [entry fileName]];
	
    }
    
    if (count == 0) {
	[countLabel setStringValue: @""];
    } else {
	NSString *label;
	label = [NSString stringWithFormat: @"remaining images: %d", count];
	[countLabel setStringValue: label];
    }

} // updateUI


- (void) readDirectory: (NSString *) fullPath
{
    BWLog (logger, BWTrace, @"reading new image directory: %@", fullPath);

    [files release];
    files = [BWFileList loadFromDirectory: fullPath  deeply: YES];
    [files sortByFileName];

    BWLog (logger, BWTrace, @"files found: %d", [files count]);

    [fileList reloadData];
    [self displayImage: [files getFileAt: 0]];
    [[fileList window] setInitialFirstResponder: fileList];
    [fileList selectRow: 0  byExtendingSelection: NO];

    // scroll to top
    [fileList scrollRowToVisible: 0];
	
} // readDirectory


- (void) awakeFromNib
{
    BWLog (logger, BWTrace, @"awakeFromNib");

    if (files != nil) {
	[self displayImage: [files getFileAt: 0]];
	[[fileList window] setInitialFirstResponder: fileList];
    }

    originalPathLabelValue = [pathLabel stringValue];

    [self updateUI];

} // awakeFromNib


- (void) dealloc
{
    BWLog (logger, BWTrace, @"dealloc");
} // dealloc


//
// ----- other Stuff
//


- (void) displayImage: (BWFileEntry *) file
{
    NSString *fullPath = [file fullPath];
    NSImage *image;
    NSSize imageSize = NSZeroSize;;

    BWLog (logger, BWTrace, @"display image:  %@", fullPath);

    image = [[NSImage alloc] initWithContentsOfFile: fullPath];

    BWLog (logger, BWVerbose, @"image: %@", image);

    imageSize = [image size];
    

    if ((image != nil) && (imageSize.width > 0) && (imageSize.height > 0)) {
	NSArray *reps = [image representations];
	NSBitmapImageRep *rep = [reps objectAtIndex: 0];
	
	imageSize.width = MAX (imageSize.width, [rep pixelsWide]);
	imageSize.height = MAX (imageSize.height, [rep pixelsHigh]);
	
	[image setSize: imageSize];
	[imageView setImage: image];
    } else {
	[imageView setImage: noImageImage];
    }

    [image release];

    [pathLabel setStringValue: fullPath];

} // displayImage



//
// ----- IB Foo
//


- (IBAction) nukeFile: (id) sender
{
    int row;
    int tag;
    NSWorkspace *workspace;
    BWFileEntry *entry;
    NSString *source;

    row = [fileList selectedRow];
    entry = [files getFileAt: row];
    source = [[entry fullPath] stringByDeletingLastPathComponent];

    BWLog (logger, BWTrace, @"nukeFile action triggered for file %@", entry);

    workspace = [NSWorkspace sharedWorkspace];

    if ([workspace performFileOperation: NSWorkspaceRecycleOperation
		   source: source
		   destination: @""
		   files: [NSArray arrayWithObject: [entry fileName]]
		   tag: &tag]) {
	BWLog (logger, BWTrace, @"move to trash succeded");
	
	// !!! known problem - dock doesn't update its trash display
	// !!! sometimes finder doesn't update trash window

	[files removeFileAt: row];
	[fileList reloadData];
	[self updateUI];
	
    } else {
	BWLog (logger, BWError, @"move to trash failed: source: %@, file: %@", 
	       source, [entry fileName]);
    }

} // nukeFile


- (IBAction) keepFile: (id) sender
{
    int row;
    BWLog (logger, BWTrace, @"keepFile action triggered");
    
    row = [fileList selectedRow];
    
    if (row == -1) {
	BWLog (logger, BWError,
	       @"internal error - keepFile action triggered with empty list selection");
	return;
    }

    if (row != -1) {
	BWBorkSortPreferences *prefs = [BWBorkSortPreferences defaultPreferences];
	NSFileManager *manager = [NSFileManager defaultManager];
	NSString *fullPath, *fileName;
	BWFileEntry *entry;
	NSString *destination = [prefs currentKeepFolder];
	BWBorkSortKeepType keepType = [prefs keepType];

	entry = [files getFileAt: row];

	BWLog (logger, BWVerbose, @"selected row %d, entry %@", row, entry);

	fullPath = [entry fullPath];
	fileName = [entry fileName];
	destination = [destination stringByAppendingFormat: @"/%@", fileName];

	if (keepType == BWCopy) {
	    if ([manager copyPath: fullPath  toPath: destination  handler: nil]) {
		BWLog (logger, BWVerbose, @"success: copy of %@ to %@",
		       fullPath, destination);
	    } else {
		BWLog (logger, BWVerbose, @"failure: copy of %@ to %@",
		       fullPath, destination);
	    }

	} else if (keepType == BWMove) {
	    if ([manager movePath: fullPath  toPath: destination  handler: nil]) {
		BWLog (logger, BWVerbose, @"success: move of %@ to %@",
		       fullPath, destination);
		[files removeFileAt: row];
		[fileList reloadData];
		[self updateUI];
	    } else {
		BWLog (logger, BWVerbose, @"failure: move of %@ to %@",
		       fullPath, destination);
	    }
	}
    }

} // keepFile


- (IBAction) setScale: (id) sender
{
    BWLog (logger, BWTrace, @"set scale: %d", [sender state]);

    if ([sender state]) {
	[imageView setImageScaling: NSScaleProportionally];
    } else {
	[imageView setImageScaling: NSScaleNone];
    }

    [imageView display];

} // setScale


- (IBAction) loadDirectory: (id) sender
{
    NSOpenPanel *panel;

    BWLog (logger, BWTrace, @"opening directory chooser");

    // set up an NSOpenPanel that'll just read directories
    panel = [NSOpenPanel openPanel];
    [panel setCanChooseDirectories: YES];
    [panel setCanChooseFiles: NO];
    [panel setAllowsMultipleSelection: NO];

    [panel beginSheetForDirectory: nil
	   file: nil
	   types: nil
	   modalForWindow: [fileList window]
	   modalDelegate: self
	   didEndSelector: @selector (openPanelDidEnd:returnCode:contextInfo:)
	   contextInfo: nil];

} // loadDirectory



- (void) openPanelDidEnd: (NSOpenPanel *) panel 
	      returnCode: (int) code  
	     contextInfo: (void *) rock
{
    NSArray *filenames;

    if (code == NSOKButton) {
	filenames = [panel filenames];

	BWLog (logger, BWTrace, @"directory chooser returned '%@'",
	       filenames);
	[self readDirectory: [filenames objectAtIndex: 0]];
    }

} // openPanelDidEnd


- (IBAction) openLogPanel: (id) sender
{
    BWLog (logger, BWTrace, @"open log panel");
} // openLogPanel



- (IBAction) showPreferences: (id) sender
{
    static BWBorkSortPrefPanel *prefPanel = nil;

    BWLog (logger, BWTrace, @"open preference panel");

    if (prefPanel == nil) {
	BWLog (logger, BWVerbose, @"created preference panel");
	prefPanel = [[BWBorkSortPrefPanel alloc] init];
    }

    [prefPanel showWindow: nil];

} // showPreferences


// turn on/off the keep and nuke items
- (BOOL) validateMenuItem: (NSMenuItem *) menuItem
{
    if (([menuItem action] == @selector(nukeFile:))
	|| ([menuItem action] == @selector(keepFile:))) {
	return ([fileList selectedRow] != -1);
    } else {
	return (YES);
    }

} // validateMenuItem

//
// ----- List View Data Source
//

- (int) numberOfRowsInTableView: (NSTableView *) view
{
    return ([files count]);
} // numberOfRowsInTableView


- (id) tableView: (NSTableView *) view
	objectValueForTableColumn: (NSTableColumn *) column
	     row: (int) rowIndex
{
    BWFileEntry *entry;
    entry = [files getFileAt: rowIndex];
    return ([entry fileName]);

} // tableView objectValueForTableColumn



//
// ----- List View delegate stuff
//

- (void) tableViewSelectionDidChange: (NSNotification *) aNotification
{
    BWLog (logger, BWVerbose, @"list selection changed");

    [self updateUI];

} // tableViewSelectionDidChange


@end // BWBorkSortController



//
// ----- evil hacks
//

@interface NSTableView (BWFirstClickHackaround)

- (BOOL) _hasEditableCell;

@end // BWFirstClickHackaround



@implementation  NSTableView (BWFirstClickHackaround)

/*
 * !!! tis is required to allow a non-text thingie to get first focus for
 * !!! a window
 */
#define TOOLKIT_WORKAROUND 1013
#if TOOLKIT_WORKAROUND
- (BOOL) _hasEditableCell
{
    return (YES);
} // _hasEditableCell
#endif

@end // BWFirstClickHackaround


